#include "runtime/mem_arena.h"
#include "runtime/resources.h"
#include "runtime/threading.h"

#include "gfx/gfx.h"
#include "gfx/renderer_api.h"

#include "asset_compiler/asset_compiler.h"

void RegisterCVars(void) {
    rtRegisterAssetCompilerCVars();
}

static rt_framegraph *_framegraph;

static rt_render_target_handle _rt;

/* Called after the runtime has finished its initialization and before entering the main-loop*/
void Init(void) {
    rtLog("GAME", "Init");
    rtInitAssetCompiler();

    rtWaitForAssetProcessing();

    _rt = g_renderer.GetSwapchainRenderTarget();
}

/* Called after exiting the main-loop and before the runtime starts its shutdown */
void Shutdown(void) {
    rtLog("GAME", "Shutdown");
    rtShutdownAssetCompiler();
}

void Update(unsigned int frame_id) {
    RT_UNUSED(frame_id);
}

void Render(unsigned int frame_id) {
    rt_alloc_command_buffer_info info = {RT_GRAPHICS_QUEUE};
    rt_command_buffer_handle cmd;
    g_renderer.AllocCommandBuffers(1, &info, &cmd);
    
    rt_cmd_begin_pass_info pass_info = {
        .color_buffer_count = 1,
        .color_buffers = {_rt},
        .color_buffer_loads = {RT_PASS_LOAD_MODE_CLEAR},
        .color_buffer_clear_values = {{.color = {1.f, 0.f, 0.f, 1.f}}},
        .name               = "testme",
    };
    g_renderer.CmdBeginPass(cmd, &pass_info);
    g_renderer.CmdEndPass(cmd);

    rt_submit_command_buffers_info submit = {.command_buffer_count = 1, .command_buffers = &cmd};
    g_renderer.SubmitCommandBuffers(RT_GRAPHICS_QUEUE, &submit);
}