#include <d3d11.h>
#include <d3d11_1.h>

#include "gfx/renderer_api.h"

#include "device_objects.hpp"
#include "gpu.hpp"

extern "C" void RT_RENDERER_API_FN(CmdBeginPass)(rt_command_buffer_handle cmdhandle,
                                                 const rt_cmd_begin_pass_info *info) {
    rt_command_buffer *cmd = rtGetCommandBuffer(cmdhandle);
    if (!RT_VERIFY(cmd))
        return;

    if (cmd->annotation) {
        WCHAR wname[128];
        if (rtUTF8ToWStr(info->name, wname, sizeof(wname)) == RT_SUCCESS)
            cmd->annotation->BeginEvent(wname);
    }

    // Setup rtvs
    ID3D11RenderTargetView *rtvs[4];
    ID3D11DepthStencilView *dsv = nullptr;

    for (uint32_t i = 0; i < info->color_buffer_count; ++i) {
        rt_render_target *rt = rtGetRenderTarget(info->color_buffers[i]);
        if (!RT_VERIFY(rt))
            return;
        RT_ASSERT(rt->IsColorRenderTarget(), "Needs to provide a valid color render target");
        rtvs[i] = rt->rtv;

        if (info->color_buffer_loads[i] == RT_PASS_LOAD_MODE_CLEAR) {
            FLOAT color[4] = {
                info->color_buffer_clear_values[i].color.r,
                info->color_buffer_clear_values[i].color.g,
                info->color_buffer_clear_values[i].color.b,
                info->color_buffer_clear_values[i].color.a,
            };
            cmd->context->ClearRenderTargetView(rt->rtv, color);
        }
    }

    rt_render_target *dsvrt = rtGetRenderTarget(info->depth_stencil_buffer);
    if (dsvrt) {
        RT_ASSERT(dsvrt->IsDepthStencilTarget(),
                  "Need to provide a valid depth stencil render target");
        dsv = dsvrt->dsv;

        if (info->depth_stencil_buffer_load == RT_PASS_LOAD_MODE_CLEAR)
            cmd->context->ClearDepthStencilView(
                dsv,
                (dsvrt->HasStencilComponent()) ? D3D11_CLEAR_DEPTH | D3D11_CLEAR_STENCIL
                                               : D3D11_CLEAR_DEPTH,
                info->depth_stencil_buffer_clear_value.depth_stencil.depth,
                static_cast<UINT8>(info->depth_stencil_buffer_clear_value.depth_stencil.stencil));
    }

    cmd->context->OMSetRenderTargets(static_cast<UINT>(info->color_buffer_count), rtvs, dsv);
}

extern "C" void RT_RENDERER_API_FN(CmdEndPass)(rt_command_buffer_handle cmdhandle) {
    rt_command_buffer *cmd = rtGetCommandBuffer(cmdhandle);
    if (!RT_VERIFY(cmd))
        return;
    if (cmd->annotation) {
        cmd->annotation->EndEvent();
    }
}

extern "C" void RT_RENDERER_API_FN(CmdTransitionRenderTarget)(rt_command_buffer_handle cmdhandle,
                                                              rt_render_target_handle target,
                                                              rt_render_target_state state) {
    rt_command_buffer *cmd = rtGetCommandBuffer(cmdhandle);
    if (!RT_VERIFY(cmd))
        return;
    RT_UNUSED(target);
    RT_UNUSED(state);
}

extern "C" void
RT_RENDERER_API_FN(CmdFlushRenderTargetWrite)(rt_command_buffer_handle cmdhandle,
                                              rt_render_target_handle render_target) {
    rt_command_buffer *cmd = rtGetCommandBuffer(cmdhandle);
    if (!RT_VERIFY(cmd))
        return;
    RT_UNUSED(render_target);
}
